<?php

// +---------------------------------------------------------------------------+
// | Ewiki Geeklog Plugin 1.0                                                  |
// +---------------------------------------------------------------------------+
// | functions-advanced.inc                                                    |
// | Search and moderation API                                                 |
// +---------------------------------------------------------------------------+
// | Plugin copyright (C) 2004 by the following author:                        |
// | Euan McKay                  -  euan at heatherengineering.com             |
// | Plugin project homepage:                                                  |
// | http://heatherengineering.com/                                            |
// | Constructed with the Universal Plugin                                     |
// | Copyright (C) 2002 by the following authors:                              |
// | Tom Willett                 -  twillett at users dot sourceforge dot net  |
// | Blaine Lang                 -  langmail at sympatico dot ca               |
// | The Universal Plugin is based on prior work by:                           |
// | Tony Bibbs                  -  tony at tonybibbs dot com                  |
// | ErfurtWiki is Public Domain                                               |
// | Written and maintained by:                                                |
// | Mario Salzer                -  mario at erphesfurt dot de                 |
// | Andy Fundinger              -  andy at burgiss dot com                    |
// | ErfurtWiki project homepage:                                              |
// | http://erfurtwiki.sourceforge.net/                                        |
// +---------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or             |
// | modify it under the terms of the GNU General Public License               |
// | as published by the Free Software Foundation; either version 2            |
// | of the License, or (at your option) any later version.                    |
// |                                                                           |
// | This program is distributed in the hope that it will be useful,           |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of            |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
// | GNU General Public License for more details.                              |
// |                                                                           |
// | You should have received a copy of the GNU General Public License         |
// | along with this program; if not, write to the Free Software Foundation,   |
// | Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.           |
// +---------------------------------------------------------------------------+

/**
* User has reuested to create a comment for the plugin
* Redirects user to comment form if initial comment for plugin record or returns
* formated HTML including the Site footer and the comments 
*
* @param        int      $comment_id      GL Comment table primary key. 
* @param        string   $commentmode     Has user selected a new comment preview mode (Flat, Nested, Threaded, None)
* @param        string   $order           Has the user selected a different comment first (Newest or Oldest comment first)
* @param        string   $reply           Has the user used the reply button on the Comment Bar 
* @return        string     Formated HTML containing site footer and the comments with the comment command bar 
*                        or redirect user to create top level comment
*/
function plugin_commentform_ewiki($comment_id,$commentmode='',$order='',$reply='') {
    global $_CONF,$_TABLES,$_FM_TABLES;
    
    // Pass the comment type - use the plugin name
       $type="ewiki";      
    // Recommended Comment ID is to use the plugin_name and a ID that links back to your plugin record.
    // In this case, I need to parse out the Plugin Record ID
    $file_id = substr($comment_id,9, strlen($comment_id-9));
    // Determine if there are existing comments for this plugin record
    $commentCount = DB_count($_TABLES['comments'],'sid',$comment_id);
    $title = DB_getItem($_FM_TABLES['ewiki'],'title',"lid=$file_id");
    // Was the comment mode passed - if not default it.
    if ($commentmode == "") {
        $commentmode = $_CONF['comment_mode'];
    }
      // Has the user selected a new comment order using the Comment Bar
    if ($order == "") {
        $order="ASC"; 
    }    
    // If this is the first comment or someone used the reply button on the comment header bar
    if ($commentCount == 0 || $reply != "") {
        $story=$comment_id;
        $pid=0;
        $type="ewiki";
        echo COM_refresh($_CONF['site_url'] . "/comment.php?sid=$comment_id&amp;pid=$pid&amp;type=$type");
    } else {
        $display = COM_siteHeader()
        // Optionally call a plugin function to return a formated view of the Plugin record so it's displayed above the comments
        $display .= plugin_commentparent_ewiki($file_id);
        // Return the formated comment records with the Comment Header and active reply links
        $display .=    COM_userComments($comment_id,$title,$type,$order,$commentmode);
        $display .= COM_siteFooter();
    } 
    return $display;
}

/**
* Check to see if this Plugin uses comments
*
* Geeklog is checking to see if this plugin supports comments, tell it no!
* NOTE: to support comments you must use the same date/time based ID for your
* widget.  In other words, to make primary keys for your plugin you should call
* makesid().  Comments are keyed off of that...it is a limitation on how geeklog
* does comments.
*
* @return   boolean true or false
*
*/
function plugin_commentsupport_ewiki() 
{
    return false;
}


/**
*
* Example Plugin Moderation Functions
* authored by Vincent Furia, http://furia.dyndns.org
*
*/

/**
*
* Checks that the current user has the rights to moderate the
* plugin, returns true if this is the case, false otherwise
*
* @return        boolean       Returns true if moderator
*
*/
function plugin_ismoderator_lists() {
    global $_USER, $_TABLES;

    return SEC_hasRights('lists.admin');
}

/**
* Returns the number of submissions waiting moderation
*
* Calculates the number of submissions that are currently waiting
* to be moderated and returns that number.
*
* @return      int       number of submissions awaiting moderation
*/
function plugin_submissioncount_lists() {
    global $_TABLES;
    return DB_count($_TABLES['listssubmission']);
}

/**
* Performs plugin exclusive work for items approved by moderation
*
* While moderation.php handles the actual move from <plugin>submission
* to <plugin> tables, within the function we handle all other approval
* relate tasks
*
* @param      string       $id      Identifying string
* @return     string       Any wanted HTML output
*
*/
function plugin_moderationapprove_lists($id) {
    global $_TABLES, $_USER, $_CONF;

    $result = DB_query("SELECT * FROM " . $_TABLES['groups'] 
                     . " WHERE grp_name = 'lists Admin'");
    $group  = DB_fetchArray($result);
    if ($_CONF['listsarchive'] == 'optional') {
        $archive = $_CONF['listsarchivedefault'];
    } elseif ($_CONF['listsarchive'] == 'no') {
        $archive = 0;
    } else {     // $_CONF['listsarchive'] == 'yes'
        $archive = 1;
    }
    $sql = "UPDATE " . $_TABLES['lists'] . " SET owner_id = " . $_USER['uid'] 
         . ", group_id = " . $group['grp_id'] . ", html = " . $_CONF['listshtml']
         . ", archive = $archive WHERE ml_id = '$id'";
    $result = DB_query($sql);

    if (DB_error()) {
       return 'Error';
    }
    return '';
}

/**
* Performs plugin exclusive work for items deleted by moderation
*
* While moderation.php handles the actual removal from <plugin>submission
* table, within this function we handle all other deletion
* relate tasks
*
* @param      string       $id      Identifying string
* @return     string       Any wanted HTML output
*
*/
function plugin_moderationdelete_lists($id) {
    global $_TABLES;

    // these tables should not contain any rows with ml_id = $id
    // this is done 'just in case'
    DB_delete($_TABLES['listsubscriptions'], 'ml_id', $id);
    DB_delete($_TABLES['listarchive'], 'ml_id', $id);
    DB_delete($_TABLES['listpermissions'], 'ml_id', $id);
}

/**
* saves submission to listssubmission table
*
* takes data input by submission form and populates the <plugin>submission
* table.
*
* @param       array       $A       array (key => value) of form items
* @return      boolean     returns true if successful
*
*/
function plugin_savesubmission_lists($A) {
    global $_TABLES, $_USER, $_CONF;

    // check for missing fields
    if (empty($A['ml_name']) || empty($A['ml_descr'])) {
        return false;
    }

    if (empty($_USER['uid'])) {
        $owner_id = 1;
    } else {
        $owner_id = $_USER['uid'];
    }

    if (SEC_hasRights('links.admin')) {
        $result = DB_getItem($_TABLES['groups'], '*', "grp_name = 'lists Admin'");
        if ($_CONF['listsarchive'] == 'optional') {
            $archive = $_CONF['listsarchivedefault'];
        } elseif ($_CONF['listsarchive'] == 'no') {
            $archive = 0;
        } else {     // $_CONF['listsarchive'] == 'yes'
            $archive = 1;
        }
        DB_save($_TABLES['lists'], 'ml_id, ml_name, ml_descr, html, archive, owner_id, group_id',
                "'" . COM_makeSid() . "', '" . $A['ml_name'] . "', '" . $A['ml_descr'] 
              . "', " . $_CONF['listshtml'] . ", $archive, $owner_id, " . $result['grp_id']);
    } elseif ($_CONF['listssubmission'] == 1) {
        DB_save($_TABLES['listssubmission'],
                'ml_id, ml_name, ml_descr',
                "'" . COM_makeSid() . "', '" . $A['ml_name'] . "', '" 
              . $A['ml_descr'] . "'");
    } else {
        return false;
    }

    if (DB_error()) {
        return false;
    }

    return true;
}

/**
* returns list of moderation values
*
* The array returned contains (in order): the row 'id' label, main plugin
* table, moderation fields (comma seperated), and plugin submission table
*
* @return       array        Returns array of useful moderation values
*
*/
function plugin_moderationvalues_lists() {
    global $_TABLES;

    return array('ml_id', $_TABLES['lists'], 'ml_id, ml_name, ml_descr'
                 , $_TABLES['listssubmission']);
}

/**
* show lists for moderation on submissions page
*
* Uses the Plugin class to return data required by moderation.php to list
* plugin objects that need to be moderated.
*
* @return       Plugin       return Plugin class containing submission data
*
*/
function plugin_itemlist_lists() {
    global $_TABLES;

    if (plugin_ismoderator_lists()) {
        $plugin = new Plugin();
        $plugin->submissionlabel = 'Mailing List Submissions';
        $plugin->getsubmissionssql = "SELECT ml_id as id, ml_name, ml_descr FROM " 
                                   . $_TABLES['listssubmission'];
        $plugin->addSubmissionHeading('List Name');
        $plugin->addSubmissionHeading('List Description');

        return $plugin;
    }
}

/**
* Returns the plugins submission form
*
* Returns a string contain the HTML of the submission form for the plugin.
*
* @return       string       The HTML form
*
*/
function plugin_submit_lists() {
    global $_CONF, $LANG12, $HTTP_POST_VARS;

    if ($_CONF['listssubmission'] == 0 && !SEC_hasRights('lists.admin')) {
        return "Submission queue disabled for mailing lists";
    }

    if ($HTTP_POST_VARS['mode'] == $LANG12[32]) { // preview
        $A = $HTTP_POST_VARS;
    $ml_id = $A['ml_id'];
    } else {
        $ml_id = COM_makesid();
    }

    $template = new Template($_CONF['path'] . "plugins/lists/templates/public");
    $template->set_file(array('form' => 'submit_form.thtml'));
    $template->set_var('site_url', $_CONF['site_url']);
    $template->set_var('lang_name', 'List Name');
    $template->set_var('ml_name', $A['ml_name']);
    $template->set_var('lang_descr', 'Description');
    $template->set_var('ml_descr', $A['ml_descr']);
    $template->set_var('ml_id', $ml_id);
    $template->set_var('lang_preview', $LANG12[32]);
    $template->set_var('lang_save', $LANG12[8]);

    return $template->parse('output', 'form');
}


?>